// Copyright 2025 "Johan Maasing" <johan@zoom.nu>
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
package nu.zoom.dsl.freemarker;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;
import nu.zoom.dsl.ast.DocumentNode;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;

public class Generator {
    private final Path templatesDir;
    private final DocumentNode data;
    private final Path outputDir;
    private final Configuration cfg;
    private final String TEMPLATE_EXTENSION = ".ftl";
    private final int TEMPLATE_EXTENSION_LENGTH = TEMPLATE_EXTENSION.length();

    public Generator(Path templatesDir, DocumentNode data, Path outputDir) throws IOException {
        this.templatesDir = Objects.requireNonNull(templatesDir);
        this.data = Objects.requireNonNull(data);
        this.outputDir = Objects.requireNonNull(outputDir);
        this.cfg = new Configuration(Configuration.VERSION_2_3_34);
        cfg.setDirectoryForTemplateLoading(templatesDir.toFile());
        cfg.setDefaultEncoding("UTF-8");
        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        cfg.setLogTemplateExceptions(false);
        cfg.setWrapUncheckedExceptions(true);
        cfg.setFallbackOnNullLoopVariable(false);
    }

    public List<Path> generate() throws IOException, TemplateException {
        try (Stream<Path> files = Files.list(templatesDir)) {
            List<String> templates = files
                    .map(Path::getFileName)
                    .map(Path::toString)
                    .filter(p -> p.length() > TEMPLATE_EXTENSION_LENGTH && p.endsWith(TEMPLATE_EXTENSION)
                    )
                    .toList();
            ArrayList<Path> out = new ArrayList<>();
            for (String template : templates) {
                Path outpath = outputDir.resolve(outputFilenameFromTemplate(template));
                Template ftl = this.cfg.getTemplate(template);
                try (var outw = Files.newBufferedWriter(outpath, StandardCharsets.UTF_8)) {
                    ftl.process(this.data, outw);
                    out.add(outpath);
                }
            }
            return out;
        }
    }

    private String outputFilenameFromTemplate(String template) {
        return template.substring(0, template.length() - TEMPLATE_EXTENSION_LENGTH);
    }
}
